@@ -1,7 +1,6 @@
 /* Functions for communicating with a remote tape drive.
 
-   Copyright (C) 1988, 1992, 1994, 1996, 1997, 1999, 2000, 2001, 2004,
-   2005, 2006, 2007 Free Software Foundation, Inc.
+   Copyright 1988-2023 Free Software Foundation, Inc.
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -37,6 +36,7 @@
 
 #include <safe-read.h>
 #include <full-write.h>
+#include <verify.h>
 
 /* Try hard to get EOPNOTSUPP defined.  486/ISC has it in net/errno.h,
    3B2/SVR3 has it in sys/inet.h.  Otherwise, like on MSDOS, use EINVAL.  */
@@ -68,10 +68,6 @@
 /* FIXME: Size of buffers for reading and writing commands to rmt.  */
 #define COMMAND_BUFFER_SIZE 64
 
-#ifndef RETSIGTYPE
-# define RETSIGTYPE void
-#endif
-
 /* FIXME: Maximum number of simultaneous remote tape connections.  */
 #define MAXUNIT	4
 
@@ -90,10 +86,10 @@ static int from_remote[MAXUNIT][2] = {{-1, -1}, {-1, -1}, {-1, -1}, {-1, -1}};
 /* The pipes for sending data to remote tape drives.  */
 static int to_remote[MAXUNIT][2] = {{-1, -1}, {-1, -1}, {-1, -1}, {-1, -1}};
 
-char *rmt_command = DEFAULT_RMT_COMMAND;
+char const *rmt_command = DEFAULT_RMT_COMMAND;
 
 /* Temporary variable used by macros in rmt.h.  */
-char *rmt_dev_name__;
+char const *rmt_dev_name__;
 
 /* If true, always consider file names to be local, even if they contain
    colons */
@@ -121,7 +117,7 @@ do_command (int handle, const char *buffer)
   /* Save the current pipe handler and try to make the request.  */
 
   size_t length = strlen (buffer);
-  RETSIGTYPE (*pipe_handler) () = signal (SIGPIPE, SIG_IGN);
+  void (*pipe_handler) (int) = signal (SIGPIPE, SIG_IGN);
   ssize_t written = full_write (WRITE_SIDE (handle), buffer, length);
   signal (SIGPIPE, pipe_handler);
 
@@ -352,6 +348,29 @@ encode_oflag (char *buf, int oflag)
   if (oflag & O_TRUNC) strcat (buf, "|O_TRUNC");
 }
 
+/* Reset user and group IDs to be those of the real user.
+   Return NULL on success, a failing syscall name (setting errno) on error.  */
+static char const *
+sys_reset_uid_gid (void)
+{
+#if !MSDOS
+  uid_t uid = getuid ();
+  gid_t gid = getgid ();
+  struct passwd *pw = getpwuid (uid);
+
+  if (!pw)
+    return "getpwuid";
+  if (initgroups (pw->pw_name, gid) != 0)
+    return "initgroups";
+  if (gid != getegid () && setgid (gid) != 0 && errno != EPERM)
+    return "setgid";
+  if (uid != geteuid () && setuid (uid) != 0 && errno != EPERM)
+    return "setuid";
+#endif
+
+  return NULL;
+}
+
 /* Open a file (a magnetic tape device?) on the system specified in
    FILE_NAME, as the given user. FILE_NAME has the form `[USER@]HOST:FILE'.
    OPEN_MODE is O_RDONLY, O_WRONLY, etc.  If successful, return the
@@ -366,8 +385,7 @@ rmt_open__ (const char *file_name, int open_mode, int bias,
   char *remote_host;		/* remote host name */
   char *remote_file;		/* remote file name (often a device) */
   char *remote_user;		/* remote user name */
-  int rc;
-  
+
   /* Find an unused pair of file descriptors.  */
 
   for (remote_pipe_number = 0;
@@ -425,23 +443,13 @@ rmt_open__ (const char *file_name, int open_mode, int bias,
 	}
   }
 
+  assume (remote_file);
+
   /* FIXME: Should somewhat validate the decoding, here.  */
-#if HAVE_GETADDRINFO
-  {
-    struct addrinfo *res;
-    rc = getaddrinfo (remote_host, NULL, NULL, &res);
-    if (rc == 0)
-      freeaddrinfo (res);
-  }
-#else
-  rc = gethostbyname (remote_host) == NULL;
-#endif
-  
-  if (rc)
-    error (EXIT_ON_EXEC_ERROR, 0,
-	   _("Cannot resolve hostname %s"), remote_host);
+  if (gethostbyname (remote_host) == NULL)
+    error (EXIT_ON_EXEC_ERROR, 0, _("Cannot connect to %s: resolve failed"),
+	   remote_host);
 
-  
   if (remote_user && *remote_user == '\0')
     remote_user = 0;
 
@@ -503,17 +511,21 @@ rmt_open__ (const char *file_name, int open_mode, int bias,
       {
 	/* Child.  */
 
-	close (STDIN_FILENO);
-	dup (to_remote[remote_pipe_number][PREAD]);
-	close (to_remote[remote_pipe_number][PREAD]);
-	close (to_remote[remote_pipe_number][PWRITE]);
+	if (dup2 (to_remote[remote_pipe_number][PREAD], STDIN_FILENO) < 0
+	    || (to_remote[remote_pipe_number][PREAD] != STDIN_FILENO
+		&& close (to_remote[remote_pipe_number][PREAD]) != 0)
+	    || (to_remote[remote_pipe_number][PWRITE] != STDIN_FILENO
+		&& close (to_remote[remote_pipe_number][PWRITE]) != 0)
+	    || dup2 (from_remote[remote_pipe_number][PWRITE], STDOUT_FILENO) < 0
+	    || close (from_remote[remote_pipe_number][PREAD]) != 0
+	    || close (from_remote[remote_pipe_number][PWRITE]) != 0)
+	  error (EXIT_ON_EXEC_ERROR, errno,
+		 _("Cannot redirect files for remote shell"));
 
-	close (STDOUT_FILENO);
-	dup (from_remote[remote_pipe_number][PWRITE]);
-	close (from_remote[remote_pipe_number][PREAD]);
-	close (from_remote[remote_pipe_number][PWRITE]);
-
-	sys_reset_uid_gid ();
+	char const *reseterr = sys_reset_uid_gid ();
+	if (reseterr)
+	  error (EXIT_ON_EXEC_ERROR, errno,
+		 _("Cannot reset uid and gid: %s"), reseterr);
 
 	if (remote_user)
 	  execl (remote_shell, remote_shell_basename, remote_host,
@@ -579,14 +591,15 @@ rmt_close__ (int handle)
 size_t
 rmt_read__ (int handle, char *buffer, size_t length)
 {
-  char command_buffer[COMMAND_BUFFER_SIZE];
+  char command_buffer[sizeof "R\n" + INT_STRLEN_BOUND (size_t)];
   size_t status;
   size_t rlen;
   size_t counter;
 
-  sprintf (command_buffer, "R%lu\n", (unsigned long) length);
+  sprintf (command_buffer, "R%zu\n", length);
   if (do_command (handle, command_buffer) == -1
-      || (status = get_status (handle)) == SAFE_READ_ERROR)
+      || (status = get_status (handle)) == SAFE_READ_ERROR
+      || status > length)
     return SAFE_READ_ERROR;
 
   for (counter = 0; counter < status; counter += rlen, buffer += rlen)
@@ -607,11 +620,11 @@ rmt_read__ (int handle, char *buffer, size_t length)
 size_t
 rmt_write__ (int handle, char *buffer, size_t length)
 {
-  char command_buffer[COMMAND_BUFFER_SIZE];
-  RETSIGTYPE (*pipe_handler) ();
+  char command_buffer[sizeof "W\n" + INT_STRLEN_BOUND (size_t)];
+  void (*pipe_handler) (int);
   size_t written;
 
-  sprintf (command_buffer, "W%lu\n", (unsigned long) length);
+  sprintf (command_buffer, "W%zu\n", length);
   if (do_command (handle, command_buffer) == -1)
     return 0;
 
@@ -639,17 +652,7 @@ rmt_write__ (int handle, char *buffer, size_t length)
 off_t
 rmt_lseek__ (int handle, off_t offset, int whence)
 {
-  char command_buffer[COMMAND_BUFFER_SIZE];
-  char operand_buffer[UINTMAX_STRSIZE_BOUND];
-  uintmax_t u = offset < 0 ? - (uintmax_t) offset : (uintmax_t) offset;
-  char *p = operand_buffer + sizeof operand_buffer;
-
-  *--p = 0;
-  do
-    *--p = '0' + (int) (u % 10);
-  while ((u /= 10) != 0);
-  if (offset < 0)
-    *--p = '-';
+  char command_buffer[sizeof "L\n0\n" + INT_STRLEN_BOUND (offset)];
 
   switch (whence)
     {
@@ -659,7 +662,8 @@ rmt_lseek__ (int handle, off_t offset, int whence)
     default: abort ();
     }
 
-  sprintf (command_buffer, "L%s\n%d\n", p, whence);
+  intmax_t off = offset;
+  sprintf (command_buffer, "L%jd\n%d\n", off, whence);
 
   if (do_command (handle, command_buffer) == -1)
     return -1;
@@ -670,7 +674,7 @@ rmt_lseek__ (int handle, off_t offset, int whence)
 /* Perform a raw tape operation on remote tape connection HANDLE.
    Return the results of the ioctl, or -1 on error.  */
 int
-rmt_ioctl__ (int handle, int operation, char *argument)
+rmt_ioctl__ (int handle, unsigned long int operation, void *argument)
 {
   switch (operation)
     {
@@ -681,24 +685,15 @@ rmt_ioctl__ (int handle, int operation, char *argument)
 #ifdef MTIOCTOP
     case MTIOCTOP:
       {
-	char command_buffer[COMMAND_BUFFER_SIZE];
-	char operand_buffer[UINTMAX_STRSIZE_BOUND];
-	uintmax_t u = (((struct mtop *) argument)->mt_count < 0
-		       ? - (uintmax_t) ((struct mtop *) argument)->mt_count
-		       : (uintmax_t) ((struct mtop *) argument)->mt_count);
-	char *p = operand_buffer + sizeof operand_buffer;
-
-        *--p = 0;
-	do
-	  *--p = '0' + (int) (u % 10);
-	while ((u /= 10) != 0);
-	if (((struct mtop *) argument)->mt_count < 0)
-	  *--p = '-';
+	struct mtop *mtop = argument;
+	enum { oplen = INT_STRLEN_BOUND (mtop->mt_op) };
+	enum { countlen = INT_STRLEN_BOUND (mtop->mt_count) };
+	char command_buffer[sizeof "I\n\n" + oplen + countlen];
 
 	/* MTIOCTOP is the easy one.  Nothing is transferred in binary.  */
 
-	sprintf (command_buffer, "I%d\n%s\n",
-		 ((struct mtop *) argument)->mt_op, p);
+	intmax_t count = mtop->mt_count;
+	sprintf (command_buffer, "I%d\n%jd\n", mtop->mt_op, count);
 	if (do_command (handle, command_buffer) == -1)
 	  return -1;
 
@@ -722,9 +717,15 @@ rmt_ioctl__ (int handle, int operation, char *argument)
 	    || (status = get_status (handle), status == -1))
 	  return -1;
 
-	for (; status > 0; status -= counter, argument += counter)
+	if (status > sizeof (struct mtop))
 	  {
-	    counter = safe_read (READ_SIDE (handle), argument, status);
+	    errno = EOVERFLOW;
+	    return -1;
+	  }
+
+	for (char *p = argument; status > 0; status -= counter, p += counter)
+	  {
+	    counter = safe_read (READ_SIDE (handle), p, status);
 	    if (counter == SAFE_READ_ERROR || counter == 0)
 	      {
 		_rmt_shutdown (handle, EIO);
@@ -737,15 +738,17 @@ rmt_ioctl__ (int handle, int operation, char *argument)
 	   than 256, we will assume that the bytes are swapped and go through
 	   and reverse all the bytes.  */
 
-	if (((struct mtget *) argument)->MTIO_CHECK_FIELD < 256)
+	struct mtget *mtget = argument;
+	if (mtget->MTIO_CHECK_FIELD < 256)
 	  return 0;
 
+	char *buf = argument;
 	for (counter = 0; counter < status; counter += 2)
 	  {
-	    char copy = argument[counter];
+	    char copy = buf[counter];
 
-	    argument[counter] = argument[counter + 1];
-	    argument[counter + 1] = copy;
+	    buf[counter] = buf[counter + 1];
+	    buf[counter + 1] = copy;
 	  }
 
 	return 0;
